// **********************************************************************
//	Le_OBJ.cpp   --   Exemplo de um programa em OpenGL para ler 
//	objetos 3D
//
//
//		 Lembre-se de adicionar as diretivas de compilao que 
//  incluem a biblioteca OPENGL
//      Le_OBJ.a -lopengl32 -lglut32 -lglu32
//
//     As teclas SETA_PARA_CIMA e SETA_PARA_BAIXO movem o observador
// **********************************************************************

#include <windows.h>
#include <stdio.h>
#include <gl\gl.h>
#include <gl\glu.h>

#ifdef __GNUC__
#include "glut.h"  
#endif

#ifndef __GNUC__
#include <gl/glut.h>
#endif

// Variveis usadas apra armazenar a possio do usurio

double UserX = 0,
	   UserY = 3,
	   UserZ = 6;

GLfloat ratio;

#include "SmVR_COBJLoader.h"

SmVR_COBJLoader *Objetos[5];


// *********************************************************************
//   ESTRUTURAS A SEREM USADAS PARA ARMAZENAR UM OBJETO 3D
// *********************************************************************

typedef struct  // Struct para armazenar um ponto
{
	float X,Y,Z;
} TPoint;



// **********************************************************************
//  void DefineLuz(void)
//
//
// **********************************************************************
void DefineLuz(void)
{
  // Define as luzes do ambiente
  GLfloat LuzAmbiente[]   = {0.2f, 0.2f, 0.2f } ;
  GLfloat LuzDifusa[]   = {0.7f, 0.7f, 0.7f, 1.0f };
  GLfloat LuzEspecular[] = {0.626281f, 0.555802f, 0.366065f, 1.0f };
  GLfloat PosicaoLuz0[]  = {3.0f, 3.0f, 0.0f, 1.0f };
  GLfloat PosicaoLuz1[]  = {-3.0f, -3.0f, 0.0f, 1.0f };
  GLfloat Especularidade[] = {1.0f, 1.0f, 1.0f, 1.0f };

   // ****************  Fonte de Luz 0

   glEnable ( GL_COLOR_MATERIAL );


   // Habilita o uso de iluminao
  glEnable(GL_LIGHTING);

  // Ativa o uso da luz ambiente
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
  // Define os parametros da Luz nmero Zero
  glLightfv(GL_LIGHT0, GL_AMBIENT, LuzAmbiente);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, LuzDifusa  );
  glLightfv(GL_LIGHT0, GL_SPECULAR, LuzEspecular  );
  glLightfv(GL_LIGHT0, GL_POSITION, PosicaoLuz0 );
  glEnable(GL_LIGHT0);

  // Ativa o "Color Tracking"
  glEnable(GL_COLOR_MATERIAL);

  // Define a reflectancia do material
  glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

  // Define a concentrao do brilho.
  // Quanto maior o valor do Segundo parametro, mais
  // concentrado ser o brilho. (Valores vlidos: de 0 a 128)
  glMateriali(GL_FRONT,GL_SHININESS,51);

  // ****************  Fonte de Luz 1

  // Ativa o uso da luz ambiente
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LuzAmbiente);
  // Define os parametros da Luz nmero Zero
  glLightfv(GL_LIGHT1, GL_AMBIENT, LuzAmbiente);
  glLightfv(GL_LIGHT1, GL_DIFFUSE, LuzDifusa  );
  glLightfv(GL_LIGHT1, GL_SPECULAR, LuzEspecular  );
  glLightfv(GL_LIGHT1, GL_POSITION, PosicaoLuz1 );
  glEnable(GL_LIGHT1);

  // Ativa o "Color Tracking"
  glEnable(GL_COLOR_MATERIAL);

  // Define a reflectancia do material
  glMaterialfv(GL_FRONT,GL_SPECULAR, Especularidade);

  // Define a concentrao do brilho.
  // Quanto maior o valor do Segundo parametro, mais
  // concentrado ser o brilho. (Valores vlidos: de 0 a 128)
  glMateriali(GL_FRONT,GL_SHININESS,20);

}

// **********************************************************************
//  void init(void)
//		Inicializa os parmetros globais de OpenGL
//
// **********************************************************************
void init(void)
{
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f); // Fundo de tela preto

	glShadeModel(GL_SMOOTH);						
	glColorMaterial ( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_DEPTH_TEST);
	glEnable ( GL_CULL_FACE );
	
}
// **********************************************************************
//  void PosicUser()
//
//
// **********************************************************************
void PosicUser()
{
	// Set the clipping volume
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(90,ratio,0.01,200);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	gluLookAt(UserX, UserY, UserZ,
		      0,0,0,
			  0.0f,1.0f,0.0f);

}
// **********************************************************************
//  void reshape( int w, int h )
//		trata o redimensionamento da janela OpenGL
//
// **********************************************************************
void reshape( int w, int h )
{

	// Prevent a divide by zero, when window is too short
	// (you cant make a window of zero width).
	if(h == 0)
		h = 1;

	ratio = 1.0f * w / h;
	// Reset the coordinate system before modifying
	glMatrixMode(GL_PROJECTION);
	//glLoadIdentity();
	// Set the viewport to be the entire window
    glViewport(0, 0, w, h);

	PosicUser();

}
// **********************************************************************
//  void DesenhaCubo()
//
//
// **********************************************************************
void DesenhaCubo()
{
	glutSolidCube(1.0);
}

// **********************************************************************
//  int IsColliding(SmVR_COBJLoader *Obj1, SmVR_COBJLoader *Obj2)
//
//
// **********************************************************************
int IsColliding(SmVR_COBJLoader *Obj1, SmVR_Matrix M1, SmVR_COBJLoader *Obj2, SmVR_Matrix M2)
{
   return Obj1->Collided(Obj2, M1, M2,0);    
}
// Posio Inicial do paraleleppedo

float PosX=0.0,PosZ= 3.5;

// **********************************************************************
//  void display( void )
//
//
// **********************************************************************
void display( void )
{
   //GLfloat Data0[4][4], Data1[4][4];
   SmVR_Matrix Data0, Data1;
	static double angY = 0;

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	DefineLuz();

	PosicUser();
	
	glMatrixMode(GL_MODELVIEW);    

	glPushMatrix();
		glTranslatef ( 1.0f, 0.0f, 0.0f );
		glRotatef(angY,0,1,0);
		glColor3f(1.0f,0.0f,0.0f);
		DesenhaCubo();		
	glPopMatrix();


	glPushMatrix();
		glTranslatef ( -1.0f, 0.0f, 0.0f );
		glRotatef(angY,0,1,0);
		glColor3f(1.0f,1.0f,0.0f);
		DesenhaCubo();		
	glPopMatrix();

	glPushMatrix();
		glTranslatef ( -3.0f, 0.0f, 0.0f );
		glRotatef(-90,0,1,0);
	    Objetos[0]->Render();
	    // Salva dados necessrio para o teste de coliso 
	    glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)Data0 );
	glPopMatrix();

    
    glPushMatrix();		
        glTranslatef ( PosX, 0.0f, PosZ );
        //glScalef(0.3,0.3,0.3);
		glRotatef(180,0,1,0);
	    Objetos[1]->Render();
	    // Salva dados necessrio para o teste de coliso
        glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)Data1 );
	glPopMatrix();

	glPushMatrix();
        glColor3f(1.0f,1.0f,1.0f);
		glTranslatef ( 0.0f, -2.0f, 0.0f );
		glScalef(5.0f,0.3f,7.0f);
        Objetos[2]->Render();        
	glPopMatrix();
	
	// realiza o teste de coliso
	int ret = IsColliding(Objetos[0], Data0, Objetos[1], Data1);
	if (ret) 
       printf ("Objetos 0 e 1 colidindo\r");
    else printf ("No colidem...                           \r");   

	angY ++;

	glutSwapBuffers();
}

// **********************************************************************
//  void keyboard ( unsigned char key, int x, int y )
//
//
// **********************************************************************
void keyboard ( unsigned char key, int x, int y )  
{
	switch ( key ) 
	{
    case 27:        // Termina o programa qdo
      exit ( 0 );   // a tecla ESC for pressionada
      break;        

    default:        
      break;
  }
}

// **********************************************************************
//  void arrow_keys ( int a_keys, int x, int y )  
//
//
// **********************************************************************
void arrow_keys ( int a_keys, int x, int y )  
{
	switch ( a_keys ) 
	{
		case GLUT_KEY_UP:       // When Up Arrow Is Pressed...
			PosZ -= 0.1;
			break;
	    case GLUT_KEY_DOWN:     // When Down Arrow Is Pressed...
			PosZ += 0.1;
			break;
		case GLUT_KEY_LEFT:       // When Up Arrow Is Pressed...
			PosX -= 0.1;
			break;
	    case GLUT_KEY_RIGHT:     // When Down Arrow Is Pressed...
			PosX += 0.1;
			
			break;
		default:
			break;
	}
}
// **********************************************************************
//  void CarregaObjetos()
//
//
// **********************************************************************
void CarregaObjetos()
{
     char nome[50];
     int Colisao=1; // 1 : vai usar teste de coliso
                    // 0 : no vai usar teste de coliso

          // Carrega um objeto "couch.obj"
     Objetos[0] = new SmVR_COBJLoader(Colisao);
     //Objetos[0] = new SmVR_COBJLoader();
     strcpy(nome, "couch.obj");
     if (!Objetos[0]->Load(nome))
     {
         printf ("Erro na Leitura do objeto '%s'\n", nome);
         system ("pause");
         exit(1);
     }
     else 
     {
          printf ("Objeto '%s' lido !\n", nome);                    
     }
    
     // Carrega um objeto "Paralelepipedo.obj"
     Objetos[1] = new SmVR_COBJLoader(Colisao);
     strcpy(nome, "Paralelepipedo.obj");
     if (!Objetos[1]->Load(nome))
     {
         printf ("Erro na Leitura do objeto '%s'\n", nome);
         system ("pause");
         exit(1);
     }
     else 
     {
          printf ("Objeto '%s' lido !\n", nome);
          // utiliza o mtodo abaixo caso seu objeto 
          // aparea "invertido" na tela
          Objetos[1]->InvertNormalOrientation();
     }
         
     // Carrega um objeto "piso.obj"
     Objetos[2] = new SmVR_COBJLoader(Colisao);
     strcpy(nome, "piso.obj");
     if (!Objetos[2]->Load(nome))
     {
         printf ("Erro na Leitura do objeto '%s'\n", nome);
         system ("pause");
         exit(1);
     }
     else 
     {
          printf ("Objeto '%s' lido !\n", nome);
     }
          
}
// **********************************************************************
//  void main ( int argc, char** argv )
//
//
// **********************************************************************
int  main ( int argc, char** argv )   
{
	glutInit            ( &argc, argv ); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_DEPTH | GLUT_RGB ); 	

	glutInitWindowPosition (0,0);
	glutInitWindowSize  ( 700, 500 ); 
	glutCreateWindow    ( "Computao Grfica - Leitura de OBJ" ); 
		
	init ();
	
	CarregaObjetos();

	glutDisplayFunc ( display );  
	glutReshapeFunc ( reshape );
	glutKeyboardFunc ( keyboard );
	glutSpecialFunc ( arrow_keys );
	glutIdleFunc ( display );

	glutMainLoop ( );          
	return 0;
}




